import request from 'supertest'
import { createApp } from '../src/createApp.js'
import { createMockServer, generateSimpleViteManifest, getRandomPort, mockConfig } from './util.js'
import fs from 'fs'
import { expect } from 'chai'

const image = fs.readFileSync('./spec/media/image.png')
const imageStat = fs.statSync('./spec/media/image.png')
const port = getRandomPort()

describe('File caching service', function () {
  let app
  let mockserver
  let restoreConfig

  before(function () {
    ;({ restore: restoreConfig } = mockConfig({ urls: [`http://localhost:${port}/`] }))
    app = createApp()
  })

  after(function () {
    restoreConfig()
  })

  beforeEach(async function () {
    mockserver = await createMockServer({ port })
    mockserver.respondWith({
      '/manifest.json': generateSimpleViteManifest({
        'example.js': { imports: ['test.txt'] },
        'test.txt': { },
        'main.css': {},
        'index.html': {
          file: 'index.html.js',
          isEntry: true,
          imports: ['example.js'],
          css: ['main.css']
        },
        'image.png': {}
      }),
      '/example.js': (req, res) => res.setHeader('content-type', 'application/javascript').status(200).send('this is example'),
      '/test.txt': (req, res) => res.setHeader('content-type', 'text/plain').status(200).send('this is test'),
      '/index.html.js': (req, res) => res.setHeader('content-type', 'application/javascript').status(200).send('this is index.html.js'),
      '/index.html': (req, res) => res.setHeader('content-type', 'text/html').status(200).send('<html><head></head><body>it\'s me</body></html>'),
      '/main.css': (req, res) => res.setHeader('content-type', 'text/css').status(200).send('.foo { color: #000; }'),
      '/favicon.ico': 'not really a favicon, though',
      '/image.png': (req, res) => {
        // need to do this like this, because jest messes up file system within tests
        res.set({
          'Content-Type': 'image/png',
          'Content-Length': imageStat.size
        })
        res.end(image)
      }
    })
    await request(app).get('/ready')
  })

  afterEach(function () {
    mockserver.close()
    process.env.CACHE_TTL = 30000
  })

  it('serves files defined in manifest.json file', async function () {
    const response = await request(app).get('/example.js')
    expect(response.statusCode).to.equal(200)
    expect(response.headers['content-type']).to.equal('application/javascript; charset=utf-8')
    expect(response.text).to.equal('this is example')
    // expect(response.headers['content-security-policy']).to.contain('sha256-NzZhMTE2Njc2YTgyNTZmZTdlZGVjZDU3YTNmYzRjNmM1OWZkMTI2NjRkYzZmMWM3YTkwMGU3ZTdhNDlhZmVlMwo=')
    const response2 = await request(app).get('/test.txt')
    expect(response2.statusCode).to.equal(200)
    expect(response2.headers['content-type']).to.equal('text/plain; charset=utf-8')
    expect(response2.text).to.equal('this is test')
  })

  it('serves css files', async function () {
    const response = await request(app).get('/main.css')
    expect(response.statusCode).to.equal(200)
    expect(response.headers['content-type']).to.equal('text/css; charset=utf-8')
    // expect(response.headers['content-security-policy']).to.contain('sha256-YjRiYWRlYTVhYmM5ZTZkNjE2ZGM4YjcwZWRlNzUxMmU0YjgxY2UxMWExOTI2ZjM1NzM1M2Y2MWJjNmUwMmZjMwo=')
  })

  it('serves / as index.html', async function () {
    const response = await request(app).get('/')
    expect(response.statusCode).to.equal(200)
    expect(response.headers['content-type']).to.equal('text/html; charset=utf-8')
    expect(response.text).to.equal('<html><head></head><body>it\'s me</body></html>')
  })

  it('adds / to dependencies', async function () {
    const response = await request(app).get('/dependencies')
    expect(response.statusCode).to.equal(200)
    const deps = JSON.parse(response.text)
    expect(deps['/']).to.deep.equal([])
  })

  it('directly fetches files not referenced in manifest.json files from the upstream servers', async function () {
    const response = await request(app).get('/favicon.ico')
    expect(response.statusCode).to.equal(200)
    expect(response.body).to.equal('not really a favicon, though')
  })

  it('returns 404 if file can not be resolved', async function () {
    const response = await request(app).get('/unknown-file.txt')
    expect(response.statusCode).to.equal(404)
  })

  it('serves binary files', async function () {
    const response = await request(app).get('/image.png')
    expect(response.statusCode).to.equal(200)
    expect(response.body.length === imageStat.size)
  })
})
