import request from 'supertest'
import { createApp } from '../src/createApp.js'
import { createMockServer, generateSimpleViteManifest, getRandomPort, mockConfig } from './util.js'
import { expect } from 'chai'

const port = getRandomPort()

describe('Responses contain custom headers', function () {
  let app
  let mockserver
  let restoreConfig

  before(function () {
    ;({ restore: restoreConfig } = mockConfig({ urls: [`http://localhost:${port}/`] }))
    app = createApp()
  })

  after(function () {
    restoreConfig()
  })

  beforeEach(async function () {
    mockserver = await createMockServer({ port })
    mockserver.respondWith({
      '/manifest.json': generateSimpleViteManifest({
        'example.js': {},
        'main.css': {},
        'index.html': {
          file: 'index.html.js',
          isEntry: true,
          imports: ['example.js'],
          css: ['main.css']
        }
      }),
      '/example.js': (req, res) => res.setHeader('content-type', 'application/javascript').status(200).send('this is example'),
      '/index.html.js': (req, res) => res.setHeader('content-type', 'application/javascript').status(200).send('this is index.html.js'),
      '/main.css': (req, res) => res.setHeader('content-type', 'text/css').status(200).send('.foo { color: #000; }'),
      '/index.html': (req, res) => res.setHeader('content-type', 'text/html').status(200).send('<html><head></head><body>it\'s me</body></html>')
    })
    await request(app).get('/ready')
  })

  afterEach(function () {
    mockserver.close()
    process.env.CACHE_TTL = 30000
  })

  it('index.html has version', async function () {
    const response = await request(app).get('/index.html')
    expect(response.statusCode).to.equal(200)
    expect(response.headers.version).to.match(/\d*\.\d*/)
  })

  it('javascript file contains dependencies', async function () {
    const response = await request(app).get('/index.html.js')
    expect(response.statusCode).to.equal(200)
    expect(response.headers.dependencies).to.equal('main.css')
  })
})
