import request from 'supertest'
import { createApp } from '../src/createApp.js'
import { createMockServer, generateSimpleViteManifest, getRandomPort, mockConfig } from './util.js'
import { expect } from 'chai'

const port = getRandomPort()

describe('Responses contain custom headers', function () {
  let app
  let mockserver
  let restoreConfig

  before(function () {
    ;({ restore: restoreConfig } = mockConfig({ urls: [`http://localhost:${port}/`] }))
    app = createApp()
  })

  after(function () {
    restoreConfig()
  })

  beforeEach(async function () {
    mockserver = await createMockServer({ port })
    mockserver.respondWith({
      '/manifest.json': generateSimpleViteManifest({
        'example.js': {}
      }),
      '/example.js': (req, res) => res.setHeader('content-type', 'application/javascript').status(200).send('this is example'),
      '/meta.json': { name: 'sample-service', version: '1.0' }
    })
    await request(app).get('/ready')
  })

  afterEach(function () {
    mockserver.close()
    delete process.env.APP_VERSION
    delete process.env.BUILD_TIMESTAMP
    delete process.env.CI_COMMIT_SHA
  })

  it('has own metadata', async function () {
    process.env.APP_VERSION = '4.2'
    process.env.BUILD_TIMESTAMP = '0123456789'
    process.env.CI_COMMIT_SHA = '0123456789abcdef'

    const response = await request(app).get('/meta')
    expect(response.statusCode).to.equal(200)
    expect(response.body).to.deep.contain({
      id: 'ui-middleware',
      name: 'UI Middleware',
      buildDate: '0123456789',
      commitSha: '0123456789abcdef',
      version: '4.2'
    })
  })

  it('has metadata from another ui service if available', async function () {
    const response = await request(app).get('/meta')
    expect(response.statusCode).to.equal(200)
    expect(response.body).to.deep.contain({
      name: 'sample-service',
      version: '1.0'
    })
  })

  it('does not have metadata from ui service when unavailable', async function () {
    await mockserver.close()

    const response = await request(app).get('/meta')
    expect(response.statusCode).to.equal(200)
    expect(response.body).to.not.deep.contain({
      name: 'sample-service',
      version: '1.0'
    })
  })
})
